angular.module('WebTrax', [
	'ngIOS9UIWebViewPatch',
	'ionic',
	'WebTrax.controllers',
	'WebTrax.services',
	'WebTrax.directives',
	'ngCordova'
])

.factory('deviceReady', function ($ionicPlatform) {
	var readyPromise;
	
	return function () {
		if (!readyPromise) {
			readyPromise = $ionicPlatform.ready();
		}
		
		return readyPromise;
	}
})

.run(function (
		deviceReady,
		$rootScope,
		$ionicLoading,
		$ionicPopup,
		$ionicHistory,
		$cordovaStatusbar,
		$cordovaKeyboard,
		$localstorage,
		$state,
		AuthServices,
		WebTraxServices,
		PushNotificationServices,
		$cordovaGoogleAnalytics,
		AppInfo,
		$q) {

	// Need to be careful: access to any of the cordova plugins must
	// be delayed until device is ready
	deviceReady().then(function() {

		// set default auto-login to ON
		var _autoLogin = $localstorage.getAutoLogin() || 'NOTSET';

		if (_autoLogin == 'NOTSET') {

			$localstorage.setAutoLogin('ON');
			console.log('Set auto-login to ON');
		}

		console.log('Auto-login is ' + _autoLogin);

		// Don't hide the accessory bar (helps with selects, especailly on iOS)
		$cordovaKeyboard.hideAccessoryBar(false)
		
		// this ensures that our "search WR/WG" fields are scrolled up when keyboard is open
		// and are not hidden behind the keyboard
		$cordovaKeyboard.disableScroll(true);


		$cordovaStatusbar.overlaysWebView(true);
		$cordovaStatusbar.style(1); //Light

		$cordovaGoogleAnalytics.debugMode();
		$cordovaGoogleAnalytics.startTrackerWithId(AppInfo.GoogleAnalyticsID);

	});
	
	$rootScope.defaultGoBack = function () {
		$ionicHistory.goBack();
	}
	
	$rootScope.goBack = $rootScope.defaultGoBack;
	
	$rootScope.handleError = function (error) {
		$cordovaGoogleAnalytics.trackException(error, true);
		$ionicLoading.hide();
		$rootScope.showAlert({title: 'Error',template:error});
	}

	$rootScope.logout = function () {
		$ionicLoading.show();
		
		$cordovaGoogleAnalytics.trackEvent('User Activities', 'Logout');

		// unregister before logout (logout resets user info)
		PushNotificationServices.unregisterDevice()
			.then(AuthServices.signOut)
			.then(function () {
				// clear cache
				WebTraxServices.cachedWgList = null;
				WebTraxServices.cachedWgId = '';
				WebTraxServices.cachedWrList = null;
				
				// clear state from app history to avoid android HW back
				$ionicHistory.nextViewOptions({
					historyRoot: true
				});
				$state.go('webtrax.login');
				$ionicLoading.hide();
			})
			.catch($rootScope.handleError);
			
			/*AuthServices.signOut().then(function(){
				// clear cache
				WebTraxServices.cachedWgList = null;
				WebTraxServices.cachedWgId = '';
				WebTraxServices.cachedWrList = null;
				
				// clear state from app history to avoid android HW back
				$ionicHistory.nextViewOptions({
					historyRoot: true
				});
				$state.go('webtrax.login');
				$ionicLoading.hide();
			});*/
	}

	// if notification arrives when previous one is still shown,
	// chaining of popups will allow showing them all one after
	// another without breaking $ionicPopup service which cannot
	// support opening a popup when another one is already shown
	$rootScope.popupPromise = $q.resolve();
	$rootScope.showAlert = function(message) {
		$rootScope.popupPromise = $rootScope.popupPromise.then(function () {
			return $ionicPopup.alert(message);
		});
		return $rootScope.popupPromise;
	}
	
	$rootScope.showConfirm = function(confirmation) {
		$rootScope.popupPromise = $rootScope.popupPromise.then(function () {
			return $ionicPopup.confirm(confirmation);
			
			});
		return $rootScope.popupPromise;
	}
	
	$rootScope.showConfirmNotify = function(confirmation) {
		$rootScope.popupPromise = $rootScope.popupPromise.then(function () {
			var confirmPopup = $ionicPopup.confirm(confirmation);
     confirmPopup.then(function(res) {
       if(res) {
       	$ionicLoading.show();
       	var wrId = confirmation.template.split(" ")[1].split("#")[1];
         $state.go('webtrax.wr-details', {'wrId': wrId}, {reload: 'webtrax.wr-details'}).then(function () {		
		})
		.catch($rootScope.handleError);
         
       } 
     });
		});
		return $rootScope.popupPromise;
	}

	// send request to check for new version (result will be checked later)
	//$rootScope.updateCheckPromise = AuthServices.updateCheck();

});

angular.module('WebTrax.controllers', []);

angular.module('WebTrax.directives', []);

angular.module('WebTrax.services', []);