angular.module('WebTrax.services')

.service('AuthServices', function($http, $q, ServiceConfig, AppInfo, $localstorage, deviceReady) {
	
	var EDS_DOMAIN = 'attdashboard.wireless.att.com';
	var EDS_COOKIE = 'attEDSEnc';

	function AuthServices() {
		// available services
		var service = {
			signIn: signIn,
			signOut: signOut,
			isAutoLoginSupported: isAutoLoginSupported,
			validAutoLoginInfo: validAutoLoginInfo,
			clearAutoLoginInfoCookie: clearAutoLoginInfoCookie,
			updateCheck: updateCheck
		};
		return service;
	}

		function updateCheck() {
		var def = $q.defer();
		$http.get(ServiceConfig.UpdateCheckURL + '/workgroups?date=' + new Date().getTime())
		.success(function (data) {
			def.resolve({
					status: 'ok'
				});
		})
		.error(function (data, status, headers, config, statusText) {
			def.resolve({
					status: 'ok'
				});
		});
		return def.promise;
	}

	/**
	 * [isAutoLoginSupported description]
	 * @return {promise} [description]
	 */
	function isAutoLoginSupported() {
		var def = $q.defer();
		if (window.cookies) {
			window.cookies.isAutoLoginSupported(function(result) {
				var _autoLoginSupported = (result === 'true');
				if (_autoLoginSupported == true) {
					def.resolve(_autoLoginSupported);
				} else {
					def.reject(_autoLoginSupported);
				}
			});
		} else {
			def.reject("window.cookies NOT AVAILABLE");
		}

		return def.promise;
	}



	/**
	 * validAutoLoginInfo check if we have a valid auto-login info for the user
	 * @param  {[String]} userName
	 * @return {[promise]}
	 */
	function validAutoLoginInfo(userName) {

		var def = $q.defer();
		if (window.cookies) {
			window.cookies.canAutoLogin(userName, function(result) {
				if (result === 'true') {
					def.resolve(result);
				} else {
					def.reject(result);
				}
			});
		} else {
			def.reject('window.cookies NOT AVAILABLE');
		}

		return def.promise;
	}


	/**
	 * clearAutoLoginInfoCookie - clear keycahain value when user disable auto-login on settings
	 * @param  {[String]} userName
	 * @return {[promise]}
	 */
	function clearAutoLoginInfoCookie(userName) {
		var def = $q.defer();

		if (window.cookies) {

			window.cookies.clearAutoLoginInfo(userName, function(result) {
				var _infoCleared = (result === 'true');
				if (_infoCleared) {
					def.resolve("CLEARED");
				} else {
					def.reject("ERROR");
				}
			});
		} else {
			def.reject("fail to clear auto login data");
		}
		return def.promise;
	}


	/**
	 * Clears all cookies
	 * @return {[promise]}
	 */
	function clearAllCookies() {
		var def = $q.defer();
		if (window.cookies) {
			window.cookies.clear(function() {
				def.resolve();
			});
		} else {
			def.resolve();
		}
		return def.promise;
	}


	/**
	 * Adds auto-login cookie
	 * @return {[promise]}
	 */
	function addAutoLoginCookie(userName) {
		var def = $q.defer();

		if (window.cookies) {
			window.cookies.addAutoLoginCookie(userName, EDS_DOMAIN, function(result) {
				var _added = (result === 'true');
				if (_added) {
					def.resolve("ADDED");
				} else {
					def.reject("ERROR");
				}
			});
		} else {
			def.reject("fail to add auto login data");
		}

		return def.promise;
	}

	/**
	 * signOut
	 */
	function signOut() {
		return clearAutoLoginInfoCookie($localstorage.getCurrentUser())
			.then(function () {
				// reset app headers
				delete $http.defaults.headers.common['Device-Type'];
				delete $http.defaults.headers.common['auth-api-token']
				delete $http.defaults.headers.common['auth-att-uid']

				//reset current user
				$localstorage.setCurrentUser('');

				return $q.resolve('OK');
			});
	}

	/**
	 * signIn
	 * @param  {[logindata with username and password for regular login or username and autologin indicator for auto-login]} logindata
	 * @return {[promise]}
	 */
	function signIn(logindata) {

		var edsAuthRequest = {
			method: 'GET',
			url: ServiceConfig.AuthURL,
			headers: {
				'attEDSApplication': 'iMoc',
				'attEDSUser': logindata.username,
				'Content-Type': undefined,
				'If-Modified-Since': 0,
				'Cache-Control': 'no-cache, no-store, must-revalidate',
				withCredentials: false
			}
		};

		//Set password header if not auto-login
		if (!logindata.autologin) {
			edsAuthRequest.headers['attEDSUserPassword'] = logindata.password;
		}

		var def = $q.defer();

		var getUserName = function () {
			return $q.when(logindata.username);
		},
		doLogin = function () {
			$http(edsAuthRequest)
			.then(function(response) {

				// set app headers
				var platform = ionic.Platform.platform() || "unknown";
				var version = ionic.Platform.version() || "unknown";
				var device = ionic.Platform.device().model || "unknown";



				$http.defaults.headers.common['Device-Type'] = device + '|' + platform + '|' + version;
				$http.defaults.headers.common['auth-api-token'] = AppInfo.ApiToken;
				$http.defaults.headers.common['auth-att-uid'] = logindata.username;


				if (ServiceConfig.bypassEdsMode) {
					$localstorage.setCurrentUser(logindata.username);
					def.resolve();
					return;
				}

				if (response.status === 200) {

					window.cookies.exists(EDS_DOMAIN, EDS_COOKIE, function(result) {

						var attEDSEncExists = (result === 'true');

						window.cookies.exists(EDS_DOMAIN, 'attEDSSec', function(result) {
							var attEDSSecExists = (result === 'true');

							if (attEDSEncExists && attEDSSecExists) {
								$localstorage.setCurrentUser(logindata.username);


								window.cookies.isAutoLoginSupported(function(result) {
									var _autoLoginSupported = (result === 'true');


									if (_autoLoginSupported == true) {

										// store in keychain and remove from cookie store, regardless of auto-login settings
										// since we need to clear the attEDSEnc cookie
										window.cookies.storeAutoLoginInfo(logindata.username, EDS_COOKIE, EDS_DOMAIN,
											function(result) {
												var autoLoginInfoSaved = (result === 'true');
												if (autoLoginInfoSaved == true) {
													def.resolve(response.data);
												} else {
													def.resolve(response.data); // failed to store data continue as usual
												}
											});


									} else {
										def.resolve(response.data); // AutoLogin NOT Supported
									}
								});

							} else {
								def.reject("Failed to Login. Security cookies are missing");
							}
						});
					});

				} else {
					def.reject("Failed to Login. " + response.statusText);
				}
			}).catch(function(error) {
				def.reject("Failed to Login. " + error.status);
			});
		};

		if (logindata.autologin) {
			clearAllCookies().then(getUserName).then(addAutoLoginCookie).then(doLogin);
		} else {
			clearAllCookies().then(doLogin);
		}

		return def.promise;
	}

	return new AuthServices();
});
