angular.module('WebTrax.controllers').controller('LoginCtrl',
  function(
    deviceReady,
    $scope,
    $rootScope,
    $state,
    $ionicSideMenuDelegate,
    AuthServices,
    $localstorage,
    $ionicHistory,
    $ionicLoading,
    AppInfo,
    $timeout,
    $q,
    PushNotificationServices,
    WebTraxServices,
    // $cordovaKeyboard,
    $cordovaGoogleAnalytics,
    $cordovaSplashscreen,
    $ionicPopup,
    $cordovaInAppBrowser) {

    var self = this;

    self.currentAppVersion = AppInfo.AppVersion;

    $ionicSideMenuDelegate.canDragContent(false); //ensures user can't see the menu on the login screen

    self.logindata = {};

    $scope.$on('$ionicView.beforeEnter', function() {
      var currentUser = $localstorage.getCurrentUser();
      self.logindata = {
        username: (currentUser || ''),
        password: '',
        autologin: false
      };
    });
		
    var year = new Date().getFullYear();

    self.copyrightText = 'Copyright © ' + year + ' Amdocs. All Rights Reserved.';

    self.signIn = function(logindata) {
      $ionicLoading.show();

      // $cordovaKeyboard.close();

      var start = Date.now();
      AuthServices.signIn(logindata)
        .then(function(response) {

          self.logindata.password = ''; //extra precaution; don't want to keep the password more than necessary

          // clear state from app history to avoid android HW back
          $ionicHistory.nextViewOptions({
            historyRoot: true
          });

          PushNotificationServices.registerDevice();

          WebTraxServices.getWgList()
            .then(function() {
              return $state.go('webtrax.wg-list');
            })
            .then(function() {
              // navigation was successful
              $cordovaGoogleAnalytics.trackView('WG List');
              $cordovaGoogleAnalytics.trackTiming('Login', Date.now() - start);

              // check for pending notification. If exists, display it and remove afterwards
              var pendingNotification = $localstorage.getObject('pending_notification');
              if (pendingNotification && pendingNotification.title) {
                //$rootScope.showConfirmNotify(pendingNotification)
                $ionicLoading.show();
                var wrId = pendingNotification.template.split(" ")[1].split("#")[1];
         					$state.go('webtrax.wr-details', {'wrId': wrId}, {reload: true}).then(function () {		
								})
								.catch($rootScope.handleError)
                  .finally(function() {
                    $localstorage.setObject('pending_notification', {});
                  });
              }
            })
            .catch($rootScope.handleError);
        })
        .catch($rootScope.handleError);
        
        $localstorage.setAutoLogin('ON');
    }

    var _processUpdateCheckResult = function() {
      var def = $q.defer();
      $rootScope.updateCheckPromise.then(function(result) {
        if (result.status == 'error') {
          console.log('App update check failed: ' + result.message);
          def.resolve();
        } else if (result.newVersion) {
          $ionicPopup.confirm({
              title: 'New Version',
              template: 'A new version is available; would you like to download it?',
              okText: 'Download'
            })
            .then(function(userChosenDownload) {
              if (userChosenDownload) {
                $cordovaInAppBrowser.open(result.updateUrl, '_system');
                ionic.Platform.exitApp();
              } else {
                def.resolve();
              }
            });
        } else {
          def.resolve();
        }
      });
      return def.promise;
    }

    // Need to be careful: access to any of the cordova plugins must
    // be delayed until device is ready
    AuthServices.updateCheck().then(function() {

          var currentUser = $localstorage.getCurrentUser();
          var _autoLogin = $localstorage.getAutoLogin();
          var _doAutoLogin = (_autoLogin == 'ON');
          _doAutoLogin = _doAutoLogin && (currentUser != '');

          if (_doAutoLogin) {

            AuthServices.isAutoLoginSupported() //if rejected, next (and following) function will not be called
              .then(function() {
                return $q.when(currentUser);
              })
              .then(AuthServices.validAutoLoginInfo) //if rejected, next (and following) function will not be called
              .then(function() {
                return $q.when({
                  username: currentUser,
                  autologin: true
                });
              })
              .then(self.signIn);
          }
      
    });

  });
