angular.module('WebTrax.controllers').controller('MenuCtrl',
function (
		$scope, 
		$rootScope,
		$ionicModal,
		$ionicLoading,
		$ionicSideMenuDelegate,
		$localstorage,
		$state,
		$stateParams,
		$ionicHistory,
		AuthServices,
		PushNotificationServices,
		WebTraxServices,
		AppInfo,
		$cordovaGoogleAnalytics) {
	var self = this;
	
	self.currentAppVersion = AppInfo.AppVersion;
	
	self.canShowSettings = false;
	
	AuthServices.isAutoLoginSupported().then(function (data) {
		self.canShowSettings = true;
	}).catch(function(err) {
		self.canShowSettings = false;
	});
	
	self.autoLoginUserSelection = $localstorage.getAutoLogin();
	
	self.logout = function () {
		$ionicSideMenuDelegate.toggleLeft(false); // close the menu

		$rootScope.logout();
	}

	var _userSettingsModal = null;
	
	$ionicModal.fromTemplateUrl('templates/settings.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		_userSettingsModal = modal;
	});

	//Cleanup the modal when we're done with it!
	$scope.$on('$destroy', function () {
		_userSettingsModal.remove();
	});
	
	self.closeAndUpdateUserSettings = function () {
		$localstorage.setAutoLogin(self.autoLoginUserSelection);

		if (self.autoLoginUserSelection == "OFF") { // clear keychain

			var currentUser = $localstorage.getCurrentUser();

			AuthServices.clearAutoLoginInfoCookie(currentUser)
			.finally(function () {
				_userSettingsModal.hide();
			});

		} else {
			_userSettingsModal.hide();
		}
	}

	self.closeUserSettings = function () {
		_userSettingsModal.hide();
	}

	self.mySettings = function () {
		self.autoLoginUserSelection = $localstorage.getAutoLogin();
	  	self.autoLoginSettings =  self.autoLoginUserSelection;

	  	$cordovaGoogleAnalytics.trackEvent('User Activities', 'Settings');

	  	_userSettingsModal.show().then(function () {
	  		// close the menu after settings modal is done animating
			$ionicSideMenuDelegate.toggleLeft(false);
	  	});
		
	}

	self.copyrightText = 'Copyright © '+ new Date().getFullYear() +' Amdocs. All Rights Reserved.';

	self.copyrightInfo =
		'\nReproduction or distribution other than for intended purposes is prohibited, ' +
		'without the prior written consent of Amdocs. The trademarks and service marks ' +
		'of Amdocs, including the Amdocs mark and logo, Intentional Customer Experience, ' +
		'CES, Clarify, Ensemble, Enabler, Return on Relationship, Intelecable, Collabrent, ' +
		'XACCT, DST Innovis, Stibo Graphic Software, Qpass, Cramer, SigValue, JacobsRimell, ' + 
		'ChangingWorlds, jNetX, OpenMarket Inc., MX Telecom Inc., MX Telecom Ltd, Streamezzo, ' +
		'and Bridgewater Systems are the exclusive property of Amdocs, and may not be used ' +
		'without permission. All other marks are the property of their respective owners.';

	self.androidNoticeForAutologin = ionic.Platform.isAndroid() ? 'Requires Android 4.3' : '';
	
	/*************** WG List ********************/
	
	self.refreshing = false;
	self.reloadingWgList = false;
	WebTraxServices.getWgList().then(function (result) {
		self.groupedWgList = result;
	});
	self.currentWgId = $stateParams.wgId;
	
	self.refreshWgList = function () {
		if (self.refreshing) {
			return;
		}
		
		self.refreshing = true;
		
		WebTraxServices.getWgList({cache:false})
		.then(function (result) {
			self.groupedWgList = result;
		})
		.catch($rootScope.handleError)
		.finally(function () {
			$scope.$broadcast('scroll.refreshComplete');
			self.refreshing = false;
		});
	}
	
	self.viewWrList = function (wgId, wgName) {
		if (self.refreshing) {
			return;
		}

		$ionicSideMenuDelegate.toggleLeft(false); // close the menu

		if (self.currentWgId != wgId) {
			$cordovaGoogleAnalytics.trackEvent('User Activities', 'Change WG');
			$ionicLoading.show();
			WebTraxServices.getWrList(wgId, {cache:false})
			.then(function () {
			
				// clear state from app history to avoid android HW back
				$ionicHistory.nextViewOptions({
					historyRoot: true
				});
				return $state.go('webtrax.wr-list', {'wgId': wgId, 'wgName': wgName});
			})
			.catch($rootScope.handleError);
		}
	}

	$rootScope.$on('webtrax.wrUpdated', function () {
		self.refreshing = true;
		WebTraxServices.getWgList({cache:false})
		.then(function (result) {
			self.groupedWgList = result;
		})
		.catch($rootScope.handleError)
		.finally(function () {
			self.refreshing = false;
		});
	})

});