angular.module('WebTrax.services')

.service('PushNotificationServices', 
		function(
			$state,
			$stateParams,
			$rootScope,
			$http,
			$q,
			ServiceConfig,
			AppInfo,
			$localstorage,
			$cordovaPushV5,
			$cordovaGoogleAnalytics) {

var push;

function PushNotifcationServices () {
	// available services
	const services = {
		registerDevice: registerDevice,
		unregisterDevice: unregisterDevice
	};
	return services;
}

function registerDevice () {
	
	$cordovaPushV5.initialize({ 
        android: {
        	senderID: AppInfo.AndroidProjectID,
            iconColor: '#0072C6'
        },
        ios : {
        	alert: true,
        	badge: false,
        	sound: true,
        	clearBadge: true
        }
    })
    .then(function () {
		$cordovaPushV5.onError(); // this will setup '$cordovaPushV5:errorOccurred' event listener
		$rootScope.$on('$cordovaPushV5:errorOccurred', function (event, err) {
			_error(err);
		});
		
		$cordovaPushV5.onNotification(); // this will setup '$cordovaPushV5:notificationReceived' event listener
		$rootScope.$on('$cordovaPushV5:notificationReceived', function (event, data) {
			var notification = {
				title: 'Notification',
				template: data.message,
        okText: 'Show Details'
			};
	
			// notification received when app is open
			if (data.additionalData && data.additionalData.foreground) {
				
				//$rootScope.showAlert(notification);
				$rootScope.showConfirmNotify(notification);
				
			} else {
				// notification received when app was in background/closed
				
				// if on login page, don't show notification until user passes login
				if ($state.current.name === 'webtrax.login') {
					$localstorage.setObject('pending_notification', notification);
				} else {
					//$rootScope.showAlert(notification);
					$rootScope.showConfirmNotify(notification);
				}
			}
		});
		
    	return $q.resolve();
    })
    .then(function () {
    	return $cordovaPushV5.register();
   	})
    .then(function (registrationId) {
    	console.log('Device token: ' + registrationId);
		
		$rootScope.deviceToken = registrationId;
		
		registerUser().then(function (response) {
			console.log('Registered device in app db');
		})
		.catch(function (err) {
			_error(err.message);
		});
    })
    .catch(function (errror) {
    	_error(error);
    });
}

function unregisterDevice () {
	var def = $q.defer();
	
	$cordovaPushV5.unregister()
	.then(function () {
		unregisterUser()
		.then (function () {def.resolve()})
		.catch (function (error) {def.reject(error)})
		.finally(function () {
			$rootScope.deviceToken = null;
		});
	});
	
	return def.promise;
}

function _error (err) {
	console.log('Push notification error: ' + err);
	$cordovaGoogleAnalytics.trackException('Push notification error: ' + err, false);
	$rootScope.showAlert({
		title: 'Notifications',
		template: 'Device registration failed. Push notifications may not work.'
	});
}

function registerUser() {
	var serviceRequest = {
		method: 'POST',
		url: ServiceConfig.ServicesURL + '/device',
		data: {
			deliveryType: ionic.Platform.isIOS() ? 'APN' : 'GCM',
			deviceToken: $rootScope.deviceToken
		},
		cache: false
	};

	var def = $q.defer();
	
	$http(serviceRequest)
	.success(function (data, status, headers, config, statusText) {
		console.log('User registered For Push notification successfully');
		def.resolve(data);
	})
	.catch(function (data, status, headers, config, statusText) {
		console.log('User registeration failed for Push Notification');
		def.reject('Failed to add device: ' + statusText);
	});

	return def.promise;
}

function unregisterUser() {
	var serviceRequest = {
		method: 'POST',
		url: ServiceConfig.ServicesURL + '/device/delete',
		data: {
			deliveryType: ionic.Platform.isIOS() ? 'APN' : 'GCM',
			deviceToken: $rootScope.deviceToken
		},
		cache: false
	};

	var def = $q.defer();
	
	$http(serviceRequest)
	.success(function (data, status, headers, config, statusText) {
		console.log('User unregistered From Push notification successfully');
		def.resolve(data);
	})
	.error(function (data, status, headers, config, statusText) {
		console.log('User unregisteration from Push Notification failed');
		def.reject('Failed to remove device: ' + statusText);
	});

	return def.promise;
}

return new PushNotifcationServices();

});