angular.module('WebTrax.controllers').controller('WgListCtrl',
function(
	$scope,
	$rootScope,
	$state,
	$stateParams,
	$ionicHistory,
	$ionicLoading,
	$localstorage,
	WebTraxServices,
	AuthServices,
	PushNotificationServices,
	$cordovaGoogleAnalytics) {

	var self = this;
	
	self.refreshing = false;
	
	var _removeWrUpdatedListenerFn = null;
	
	WebTraxServices.getWgList().then(function (result) {
		self.groupedWgList = result;
	});
	
	self.logout = function () {
		// clean up the scope variables before logging out
		$scope.wrId = '';
		$rootScope.logout();
	}

	self.refreshWgList = function () {
		if (self.refreshing) {
			return;
		}
		
		self.refreshing = true;
		var start = Date.now();
		WebTraxServices.getWgList({cache:false})
		.then(function (result) {
			self.groupedWgList = result;
			$cordovaGoogleAnalytics.trackTiming('Refresh WG List', Date.now() - start);
		})
		.catch($rootScope.handleError)
		.finally(function () {
			$scope.$broadcast('scroll.refreshComplete'); // tell ionic to hide the refreshing indicator
			self.refreshing = false;
		});
	}
	
	self.viewWrList = function (wgId, wgName) {
		
		if (self.refreshing) {
			return;
		}
		
		$ionicLoading.show();
		var start = Date.now();
		WebTraxServices.getWrList(wgId)
		.then(function () {
			// clear state from app history to avoid android HW back
			$ionicHistory.nextViewOptions({
				historyRoot: true
			});
			return $state.go('webtrax.wr-list', {'wgId': wgId, 'wgName': wgName})
		})
		.then(function () {
			// navigation was successful
			$cordovaGoogleAnalytics.trackTiming('WR List', Date.now() - start);
			
			if (_removeWrUpdatedListenerFn) {
				_removeWrUpdatedListenerFn();
				_removeWrUpdatedListenerFn = null;
			}
			
		})
		.catch($rootScope.handleError);
	}
	
	self.searchWr = function (wrId) {
		if (self.refreshing) {
			return;
		}

		$cordovaGoogleAnalytics.trackEvent('User Activities', 'Search WR from WG List');
		
		$ionicLoading.show();
		var start = Date.now();
		$state.go('webtrax.wr-details', {'wrId': wrId})
		.then(function () {
			$cordovaGoogleAnalytics.trackView('WR Details');
			$cordovaGoogleAnalytics.trackTiming('WR Details', Date.now() - start);
		})
		.catch($rootScope.handleError);
	}

	// if user open WR from Workgroups screen and makes update to WR we need
	// to refresh workgroups lists when coming back. When user continues to WR list screen
	// this listener will be removed because user cannot go back to Workgroups screen
	_removeWrUpdatedListenerFn = $rootScope.$on('webtrax.wrUpdated', function () {
		self.refreshing = true;
		WebTraxServices.getWgList({cache:false})
		.then(function (result) {
			self.groupedWgList = result;
		})
		.catch($rootScope.handleError)
		.finally(function () {
			self.refreshing = false;
		});
	})
});