angular.module('WebTrax.controllers').controller('WrDetailsCtrl',
function(
	$scope,
	$rootScope,
	$state,
	$stateParams,
	$ionicModal,
	$ionicScrollDelegate,
	$ionicHistory,
	$ionicLoading,
	$ionicPlatform,
	$localstorage,
	WebTraxServices,
	wrDetails,
	UserInfo,
	$cordovaGoogleAnalytics,
	$cordovaDatePicker) {

	var self = this;

	self.refreshing = false;

	self.editing = false;

	self.wrDetails = {};
	var originalWrDetails = {};
	self.lastUpdated = new Date();
	
	var _removeCustomGoBackFn = null;
	
	/********************** WG Selection Dialog ******************************************/
	
	var _selectWorkgroupModal = null;
	
	self.filteredWgList = [];
	
	self.selectedWorkgroup = null;

	self.groupSearchText = '';

	$ionicModal.fromTemplateUrl('templates/select-wg.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		_selectWorkgroupModal = modal;
	});

	//Cleanup the modal when we're done with it!
	$scope.$on('$destroy', function () {
		_selectWorkgroupModal.remove();
	});

	self.selectWorkgroup = function () {
		
		//reset group's expanded/collapsed states before opening the modal

		//Assigned is expanded by default
		self.wrDetails.viewMobGrpsListInWrDetlPage[0].expanded = true;
		//Touched is expanded by default
		self.wrDetails.viewMobGrpsListInWrDetlPage[1].expanded = true;
		//Reset of the groups are collapsed by default
		for (var i = 2; i < self.wrDetails.viewMobGrpsListInWrDetlPage.length; ++i) {
			self.wrDetails.viewMobGrpsListInWrDetlPage[i].expanded = false;
		}
		
		//reset search text
		self.groupSearchText = '';
		
		//reset selected group
		self.selectedWorkgroup = {
			groupId: self.wrDetails.assignedGroup.groupId,
			groupName: self.wrDetails.assignedGroup.groupName
		};
		
		_buildFilteredWgList();
		
		$cordovaGoogleAnalytics.trackView('WG Selection Dialog');
		_selectWorkgroupModal.show();
	}
	
	/**
	 * To speed-up the ng-repeats we're using "2 lists" pattern; all filtering and collapsing/expanding
	 * will be applied on the source list and produce the list that will be used in the ng-repeat
	 * 
	 * the items in filtered list must have unique and stable IDs, but same group may appear in multiple group types
	 * so we can't just use groupId. Additionally, group types don't have IDs at all  
	 */
	var _buildFilteredWgList = function () {
		var list = [];
		angular.forEach(self.wrDetails.viewMobGrpsListInWrDetlPage, function (groupType, typeIndex) {
			
			// create list item for the group type
			
			var groupTypeListItem = {
				trackingId: 1000 * (typeIndex + 1), // generating tracking ID; assuming less than 1000 group in each groupType
				typeIndex: typeIndex,
				groupType: groupType.groupType,
				expanded: groupType.expanded
			};
			
			// go over groups and create list items; if group search is activated, apply group name filtering
			var groupListItems = [];
			var searchActive = self.groupSearchText != '';
			var searchRegExp = searchActive ? new RegExp(self.groupSearchText, 'i') : null;
			
			angular.forEach(groupType.groupList, function (group, groupIndex) {
				if (!searchActive || (searchActive && searchRegExp.test(group.groupName))) {
					groupListItems.push({
						trackingId: groupTypeListItem.trackingId + groupIndex + 1,  // generating tracking ID; adding 1 so the first item does not get the same ID as parent group type
						groupId: group.groupId,
						groupName: group.groupName,
						selected: (self.selectedWorkgroup && group.groupId == self.selectedWorkgroup.groupId)
					});
				}
			});
			
			// adding group type to the final list only if it has at least one group
			if (groupListItems.length > 0) {
				list.push(groupTypeListItem);
			}
			
			// adding groups to the final list only if the group is expanded
			if (groupTypeListItem.expanded) {
				angular.forEach(groupListItems, function (group) {
					list.push(group);
				});
			}

		});
		
		// replace the filtered list with the new one
		self.filteredWgList = list;
	}
	
	self.groupListItemClicked = function (group) {
		if (group.groupType) {
			// flip the expanded property using the right index
			self.wrDetails.viewMobGrpsListInWrDetlPage[group.typeIndex].expanded = !self.wrDetails.viewMobGrpsListInWrDetlPage[group.typeIndex].expanded;
		} else if (group.groupName) {
			self.selectedWorkgroup = group;
		}
		_buildFilteredWgList();
	}

	self.closeWorkgroupSelection = function () {
		_selectWorkgroupModal.hide();
	}
	
	self.closeAndSelectWorkgroup = function () {
		// get the request types for selected workgroup before closing the modal
		$ionicLoading.show();
		WebTraxServices.getWrTypeList(self.selectedWorkgroup.groupId)
		.then(function (result) {
			self.wrDetails.wrTypesList = result;
			if (self.wrDetails.wrType) {
				var wrTypeExists = false;
				for (var i = 0; i < result.length; ++i) {
					if (self.wrDetails.wrType.wrTypeId == result[i].wrTypeId) {
						wrTypeExists = true;
						break;
					}
				}
				if (!wrTypeExists) {
					self.wrDetails.wrType = null;
				}
			}
			
			_selectWorkgroupModal.hide();
			self.wrDetails.assignedGroupId = self.selectedWorkgroup.groupId;
			self.wrDetails.assignedGroup = self.selectedWorkgroup;
		})
		.catch($rootScope.handleError)
		.finally(function () {
			$ionicLoading.hide();
		});

	}

	self.groupSearchTextChanged = function () {
		_buildFilteredWgList();
		$ionicScrollDelegate.$getByHandle('selectWgList').resize();
	}

	self.resetWorkgroupSearch = function () {
		self.groupSearchText = '';
		self.groupSearchTextChanged();
	}

	self.canChangeWorkgroup = function () {
		return self.wrDetails.viewMobGrpsListInWrDetlPage.length > 0;
	}
	
	self.userChangedWorkgroup = function () {
		return self.wrDetails.assignedGroup.groupId != originalWrDetails.assignedGroup.groupId;
	}
	
	/********************** User Selection Dialog ******************************************/

	var _selectUserModal = null;
	
	self.filteredUserList = [];
	
	self.selectedUser = null;

	self.userSearchText = '';

	$ionicModal.fromTemplateUrl('templates/select-user.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function (modal) {
		_selectUserModal = modal;
	});

	//Cleanup the modal when we're done with it!
	$scope.$on('$destroy', function () {
		_selectUserModal.remove();
	});
	
	self.selectUser = function () {
		
		//reset search text
		self.userSearchText = '';
		
		//reset selected user
		self.selectedUser = null;
		
		_buildFilteredUserList();
		
		$cordovaGoogleAnalytics.trackView('User Selection Dialog');
		_selectUserModal.show();
	}

	/**
	 * To speed-up the ng-repeats we're using "2 lists" pattern; all filtering will be applied on the source list
	 * and produce the list that will be used in the ng-repeat
	 * 
	 */
	var _buildFilteredUserList = function () {
		var list = [];
		
		var searchActive = self.userSearchText != '';
		var searchRegExp = searchActive ? new RegExp(self.userSearchText, 'i') : null;
		
		angular.forEach(self.wrDetails.assignedUsers, function (user) {
			if (!searchActive || (searchActive && (searchRegExp.test(user.fullName) || searchRegExp.test(user.suitsId)))) {
				user.selected = self.selectedUser && self.selectedUser.suitsId == user.suitsId;
				list.push(user);
			}
		});
		
		// replace the filtered list with the new one
		self.filteredUserList = list;
	}
	
	self.userListItemClicked = function (user) {
		self.selectedUser = user;
		_buildFilteredUserList();
	}

	self.closeUserSelection = function () {
		_selectUserModal.hide();
	}
	
	self.closeAndSelectUser = function () {
		_selectUserModal.hide();
		self.wrDetails.assignedAnalyst = self.selectedUser.suitsId;
		self.wrDetails.assignedAnalystName = self.selectedUser.fullName + ' (' + self.selectedUser.suitsId + ')';
	}

	self.userSearchTextChanged = function () {
		_buildFilteredUserList();
		$ionicScrollDelegate.$getByHandle('selectUserList').resize();
	}

	self.resetUserSearch = function () {
		self.userSearchText = '';
		self.userSearchTextChanged();
	}
	
	self.canChangeAnalyst = function () {
		return self.wrDetails.assignedUsers.length > 0 && !self.userChangedWorkgroup();
	}
	
	/********************** Due and ETR Dates ******************************************/
	
	self.canChangeDueDate = function () {
		return self.wrDetails.canChangeDueDate;
	}
	
	self.selectDueDate = function () {
		var options = {
			date: self.wrDetails.dueDate ? self.wrDetails.dueDate : new Date(),
			mode: 'date'
		};
		
		$cordovaDatePicker.show(options).then(function (date) {
			self.wrDetails.dueDate = date;
		});
	}
	
	self.canChangeETRDate = function () {
		return self.wrDetails.canChangeETRDate;
	}
	
	self.selectETRDate = function () {
		var options = {
			date: self.wrDetails.etrDate ? self.wrDetails.etrDate : new Date(),
			mode: 'date'
		};
		
		$cordovaDatePicker.show(options).then(function (date) {
			self.wrDetails.etrDate = date;
		});
	}

	/*******************************************************************************/
	
	function _setWrDetails(data) {
		self.wrDetails = data;
		originalWrDetails = angular.copy(data);
		self.lastUpdated = new Date();
		// if spiCheck is set, spiInd will be disabled so we need to set spiInd from spiCheck
		self.wrDetails.spiInd = data.spiCheck > 0 ? 1 : 0;
	}
	
	function _doRefresh() {
		self.editing = false;
		self.refreshing = true;
		
		var start = Date.now();
		WebTraxServices.getWrDetails($stateParams.wrId)
		.then(function (result) {
			_setWrDetails(result);
			self.lastUpdated = new Date();
			$cordovaGoogleAnalytics.trackTiming('Refresh WR Details', Date.now() - start);
		})
		.catch($rootScope.handleError)
		.finally(function () {
			$scope.$broadcast('scroll.refreshComplete'); // tell ionic to hide the refreshing indicator 
			self.refreshing = false;
		});
	}
	
	self.refreshWrDetails = function () {
		if (self.refreshing) {
			return;
		}

		if (self.editing) {
			$rootScope.showConfirm({
				title: 'Edit Mode',
				template: 'Any unsaved changes will be lost. Continue?',
				cancelText: 'No',
				okText: 'Yes'
			}).then(function (res) {
				if (res) {
					_doRefresh();
				} else {
					// just hide the refreshing indicator
					$scope.$broadcast('scroll.refreshComplete');
				}
			});
		} else {
			_doRefresh();
		}
	}
	
	function _loadWrDetails () {
		$ionicLoading.show();

		WebTraxServices.getWrDetails($stateParams.wrId)
		.then(function (result) {
			_setWrDetails(result);
			self.lastUpdated = new Date();
		})
		.catch($rootScope.handleError)
		.finally(function () {
			$ionicLoading.hide();
		});
	}
	
	function _wrDetailsGoBack () {
		if (!self.editing) {
			$ionicHistory.goBack();
		} else {
			$rootScope.showConfirm({
				title: 'Edit Mode',
				template: 'Any unsaved changes will be lost. Continue?',
				cancelText: 'No',
				okText: 'Yes'
			}).then(function (res) {
				if (res) {
					$cordovaGoogleAnalytics.trackEvent('User Activities', 'Leaving WR Edit Mode Without Saving');
					$ionicHistory.goBack();
				}
			});
		}
	}

	// overriding the default back action before loading the WR details view
	$scope.$on('$ionicView.beforeEnter', function () {
		_setWrDetails(wrDetails);
		$rootScope.goBack = _wrDetailsGoBack;
		_removeCustomGoBackFn = $ionicPlatform.registerBackButtonAction(_wrDetailsGoBack, 101);
	});
	
	// restoring the default back action after leaving the WR details view
	$scope.$on('$ionicView.afterLeave', function () {
		$rootScope.goBack = $rootScope.defaultGoBack;
		if (_removeCustomGoBackFn) {
			_removeCustomGoBackFn();
			_removeCustomGoBackFn = null;
		}
	});

	self.edit = function () {
		// if editRequestRequired is true we must to call 'getWrDetails' with editMode=true
		// the backend may auto-assign this WR to the current user
		if (self.wrDetails.editRequestRequired) {
			$ionicLoading.show();

			WebTraxServices.getWrDetails($stateParams.wrId, true)
			.then(function (result) {
				_setWrDetails(result);
				$ionicLoading.hide();
				if (self.wrDetails.wrAutoAssignFlag) {
					$rootScope.showAlert({title: 'Warning', template: 'This WR has been auto assigned to ' + self.wrDetails.assignedAnalystName})
				}
				self.editing = true;
			})
			.catch($rootScope.handleError);
		} else {
			self.editing = true;
		}
	}

	self.save = function () {
		if (self.refreshing) {
			return;
		}
		
		if (!self.wrDetails.wrType) {
			$rootScope.showAlert({title: 'Error', template:'Please select request type'});
			return;
		}

		$ionicLoading.show();

		var start = Date.now();

		if (originalWrDetails.assignedGroup.groupId != self.wrDetails.assignedGroup.groupId) {
			$cordovaGoogleAnalytics.trackEvent('Save WR', 'Input', 'New Workgroup');
		}
		if (originalWrDetails.status.statusId != self.wrDetails.status.statusId) {
			$cordovaGoogleAnalytics.trackEvent('Save WR', 'Input', 'New Status');
		}
		if (originalWrDetails.priority.priorityId != self.wrDetails.priority.priorityId) {
			$cordovaGoogleAnalytics.trackEvent('Save WR', 'Input', 'New Priority');
		}
		if ((!originalWrDetails.assignedAnalyst && self.wrDetails.assignedAnalyst) ||
			(originalWrDetails.assignedAnalyst && originalWrDetails.assignedAnalyst != self.wrDetails.assignedAnalyst)) {
			$cordovaGoogleAnalytics.trackEvent('Save WR', 'Input', 'New Analyst');
		}
		if ((!originalWrDetails.wrType && self.wrDetails.wrType) ||
			(originalWrDetails.wrType && self.wrDetails.wrType && originalWrDetails.wrType.wrTypeId != self.wrDetails.wrType.wrTypeId)) {
			$cordovaGoogleAnalytics.trackEvent('Save WR', 'Input', 'New Request Type');
		}
		if ((!originalWrDetails.dueDate && self.wrDetails.dueDate) ||
			(originalWrDetails.dueDate && self.wrDetails.dueDate && originalWrDetails.dueDate !== self.wrDetails.dueDate)) {
			$cordovaGoogleAnalytics.trackEvent('Save WR', 'Input', 'New Due Date');
		}
		if ((!originalWrDetails.etrDate && self.wrDetails.etrDate) ||
			(originalWrDetails.etrDate && self.wrDetails.etrDate && originalWrDetails.etrDate !== self.wrDetails.etrDate)) {
			$cordovaGoogleAnalytics.trackEvent('Save WR', 'Input', 'New ETR Date');
		}
		

		WebTraxServices.saveWr(self.wrDetails)
		.then(function (result) {
			$cordovaGoogleAnalytics.trackEvent('Save WR', 'Result', 'Success');
			$cordovaGoogleAnalytics.trackTiming('Save WR', Date.now() - start);
			$ionicLoading.hide();
			$rootScope.showAlert({title: 'Success', template: result.statusMessage})
			.then(function () {
				//trigger wrUpdate after goBack finishes
				var offState = $rootScope.$on('$stateChangeSuccess', 
	            function(event, toState, toParams, fromState, fromParams){
					$rootScope.$broadcast('webtrax.wrUpdated');
	                offState();
	            });
				$ionicHistory.goBack();
				
			});
		})
		.catch(function (error) {
			$ionicLoading.hide();
			$rootScope.showAlert({title: 'Error', template:error.statusMessage})
			.then(function () {
				//409 status code means WR is stale (has been updated by another user)
				if (error.statusCode === '409') {
					$cordovaGoogleAnalytics.trackEvent('Save WR', 'Result', 'Stale WR');
					_loadWrDetails();
				} else {
					$cordovaGoogleAnalytics.trackEvent('Save WR', 'Result', 'Error');
					$cordovaGoogleAnalytics.trackException(error.statusMessage, false);
				}
			});
		});
	}

});