angular.module('WebTrax.controllers').controller('WrListCtrl',
function(
	$scope,
	$rootScope,
	$state,
	$stateParams,
	$ionicLoading,
	$ionicScrollDelegate,
	$ionicActionSheet,
	$filter,
	WebTraxServices,
	$cordovaGoogleAnalytics) {

	var self = this;
	
	self.refreshing = false;
	
	self.currentWgId = $stateParams.wgId;
	self.wgName = $stateParams.wgName;
	WebTraxServices.getWrList(self.currentWgId).then(function (result) {
		self.wrListData = result;
		_initPriorityFilters();
	});
	
	var _allPriorities = null;
	
	function _filterItemClicked (index, item) {
		$cordovaGoogleAnalytics.trackEvent('User Activities', 'Filter WR List');
		$ionicScrollDelegate.$getByHandle('wrList').resize();

		item.selected = !item.selected;
		item.text = '<span class="' + self.getPriorityClass(item.priorityId) + '"><i class="icon ';
		if (item.selected) {
			item.text += 'ion-checkmark';
		}
		item.text += '"></i>' + item.priorityName;
		item.text += '</span>';
		return false;
	}
	
	function _initPriorityFilters () {
		_allPriorities = [];
		var prioritiesFromWrList = {};
		angular.forEach(self.wrListData.wrList, function (wr) {
			prioritiesFromWrList[wr.wrPriorityName] = wr.priorityId;
		});
		
		angular.forEach(prioritiesFromWrList, function (priorityId, priorityName) {
			_allPriorities.push({
				priorityId: priorityId,
				priorityName: priorityName,
				selected: true,
				text:	'<span class="' + self.getPriorityClass(priorityId) + '"><i class="icon ion-checkmark"></i>' + priorityName + '</span>'
			});
		});
		
		_allPriorities = $filter('orderBy')(_allPriorities, 'priorityId');
	}

	self.getPriorityClass = function (priorityId) {
		switch (priorityId) {
			case '1':
				return 'assertive';
			case '2':
				return 'energized';
			case '3':
				return 'royal';
			default:
				return 'calm';
		}
	}
	
	self.showFilterSelection = function () {
		$cordovaGoogleAnalytics.trackEvent('User Activities', 'Show WR Filters');
		
		$ionicActionSheet.show({
			buttons: _allPriorities,
			buttonClicked: _filterItemClicked,
			cssClass: 'filter-selection'
		});
	}
	
	self.applyPriorityFilter = function (item) {
		var result = false;
		angular.forEach(_allPriorities, function (filter){
			if (filter.priorityId === item.priorityId) {
				result = filter.selected;
			}
		});
		return result;
	}
	
	self.refreshWrList = function () {
		if (self.refreshing) {
			return;
		}
		
		$cordovaGoogleAnalytics.trackEvent('User Activities', 'Refresh WR List');
		
		self.refreshing = true;
		var start = Date.now();
		WebTraxServices.getWrList($stateParams.wgId, {cache:false})
		.then(function (result) {
			$cordovaGoogleAnalytics.trackTiming('Refresh WR List', Date.now() - start);
			self.wrListData = result;
			_initPriorityFilters();
		})
		.catch($rootScope.handleError)
		.finally(function () {
			$scope.$broadcast('scroll.refreshComplete'); // tell ionic to hide the refreshing indicator
			self.refreshing = false;
		});
	}
	
	self.viewWrDetails = function (wrId) {
		if (self.refreshing) {
			return;
		}
		
		$cordovaGoogleAnalytics.trackEvent('User Activities', 'Select WR');
		
		$ionicLoading.show();
		var start = Date.now();
		$state.go('webtrax.wr-details', {'wrId': wrId})
		.then(function () {
			// navigation was successful
			$cordovaGoogleAnalytics.trackView('WR Details');
			$cordovaGoogleAnalytics.trackTiming('WR Details', Date.now() - start);
			delete $scope.wrId;
		})
		.catch($rootScope.handleError);
	}

	self.searchWr = function (wrId) {
		if (self.refreshing) {
			return;
		}
		
		$cordovaGoogleAnalytics.trackEvent('User Activities', 'Search WR from WR List');

		$ionicLoading.show();
		var start = Date.now();
		$state.go('webtrax.wr-details', {'wrId': wrId})
		.then(function () {
			// navigation was successful
			$cordovaGoogleAnalytics.trackView('WR Details');
			$cordovaGoogleAnalytics.trackTiming('WR Details', Date.now() - start);
			delete $scope.wrId;
		})
		.catch($rootScope.handleError);
	}

	// if user makes update to WR we need to refresh WR lists when coming back.
	$rootScope.$on('webtrax.wrUpdated', function () {
		$ionicLoading.show();
		WebTraxServices.getWrList(self.currentWgId, {cache:false})
		.then(function (result) {
			self.wrListData = result;
			_initPriorityFilters();
		})
		.catch($rootScope.handleError)
		.finally(function () {
			$ionicLoading.hide();
		});
	})
});