/*
 * Copyright 2013 Ernests Karlsons
 * https://github.com/bez4pieci
 * http://www.karlsons.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "CDVCookies.h"
#import <Security/Security.h>


@implementation CDVCookies


- (void)clear:(CDVInvokedUrlCommand*)command
{
    NSHTTPCookie *cookie;
    NSHTTPCookieStorage *storage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
    for (cookie in [storage cookies]) {
        [storage deleteCookie:cookie];
    }
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    CDVPluginResult* pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}


- (void)exists:(CDVInvokedUrlCommand*)command
{
    BOOL _exists = FALSE;
    NSString *cookieDomain = [command argumentAtIndex:0];
    NSString *cookieName = [command argumentAtIndex:1];
    
    CDVPluginResult* pluginResult = nil;
    
    if (!cookieDomain || !cookieName) {
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"no parameters given"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:[command callbackId]];
        return;
    }
    
    
    
    
    NSHTTPCookie *cookie;
    NSHTTPCookieStorage *storage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
    for (cookie in [storage cookies]) {
        if ([[cookie name] isEqualToString:cookieName ]  && [[cookie domain] isEqualToString:cookieDomain ]) {
            _exists = TRUE;
            break;
        }
    }
    
    pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString: ( _exists ? @"true" : @"false" ) ];
    
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}




- (void)canAutoLogin:(CDVInvokedUrlCommand *)command
{
    
    NSString *userName = [command argumentAtIndex:0];
    CDVPluginResult* pluginResult = nil;
    
    if (!userName) {
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"no parameter given"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:[command callbackId]];
        return;
    }
    
    
    // check if exists in keychain
    
    BOOL _secureCookieExists = [self existsInKeyChain:userName];
    
    pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString: ( _secureCookieExists ? @"true" : @"false" ) ];
    
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
    
}


- (void)storeAutoLoginInfo:(CDVInvokedUrlCommand *)command
{
    NSString *userName = [command argumentAtIndex:0];
    NSString *cookieName = [command argumentAtIndex:1];
    NSString *cookieDomain = [command argumentAtIndex:2];
    
    
    CDVPluginResult* pluginResult = nil;
    
    if (!userName || !cookieName || !cookieDomain) {
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"no parameter given"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
        return;
    }
    
    
    // read the cookie from cookie store
    // search for the remeber me cookie
    
    NSData* attEDSEncData;
    BOOL _storedInKeyChain = NO;
    
    NSHTTPCookie *cookie;
    NSHTTPCookieStorage *storage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
    for (cookie in [storage cookies]) {
        if ([[cookie name] isEqualToString:cookieName ]  && [[cookie domain] isEqualToString:cookieDomain ]) {
            attEDSEncData = [NSKeyedArchiver archivedDataWithRootObject:cookie.properties];
            break;
        }
    }
    
    
    
    
    OSStatus status;
    
    if (attEDSEncData != nil) {
        
        // store attEDSEnc in keyChain
        if ([self existsInKeyChain:userName]) {
            // we need to update the entry
            NSDictionary* attEDSEncQuery = @{
                                             (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
                                             (__bridge id)kSecAttrService : @"attEDSEnc",
                                             (__bridge id)kSecAttrAccount : userName
                                             };
            NSDictionary* attEDSEncChanges = @{
                                               (__bridge id)kSecValueData : attEDSEncData
                                               };
            
            status = SecItemUpdate((__bridge CFDictionaryRef)attEDSEncQuery, (__bridge CFDictionaryRef)attEDSEncChanges);
            
        } else { // create a new entry
            NSDictionary* attEDSEncQueryNew = @{
                                                (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
                                                (__bridge id)kSecAttrService : @"attEDSEnc",
                                                (__bridge id)kSecAttrAccount : userName,
                                                (__bridge id)kSecValueData : attEDSEncData
                                                };
            
            status = SecItemAdd((__bridge CFDictionaryRef)attEDSEncQueryNew, NULL);
        }
        
        _storedInKeyChain = !status;
    }
    
    
    
    // remove from cookie store
    
    for (cookie in [storage cookies]) {
        if ([[cookie name] isEqualToString:cookieName ]  && [[cookie domain] isEqualToString:cookieDomain ]) {
            [storage deleteCookie:cookie];
            break;
        }
        
    }
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    
    pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString: ( _storedInKeyChain ? @"true" : @"false" ) ];
    
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
    
}



- (BOOL)existsInKeyChain:(NSString*)userName
{
    NSDictionary* attEDSEncExistsQuery = @{
                                           (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
                                           (__bridge id)kSecAttrService : @"attEDSEnc",
                                           (__bridge id)kSecAttrAccount : userName,
                                           (__bridge id)kSecReturnData : @NO
                                           };
    
    CFDataRef attributes = nil;
    OSStatus status = SecItemCopyMatching((__bridge CFDictionaryRef)attEDSEncExistsQuery, (CFTypeRef*)&attributes);
    return !status;
    
}




/**
 *
 * clear user autoLoginInfo
 *
 */

- (void)clearAutoLoginInfo:(CDVInvokedUrlCommand *)command
{
    
    NSString *userName = [command argumentAtIndex:0];
    CDVPluginResult* pluginResult = nil;
    
    if (!userName) {
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"no parameter given"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:[command callbackId]];
        return;
    }
    
    
    NSDictionary* attEDSEncQuery = @{
                                     (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
                                     (__bridge id)kSecAttrService : @"attEDSEnc",
                                     (__bridge id)kSecAttrAccount : userName
                                     };
    
    OSStatus status = SecItemDelete((__bridge CFDictionaryRef)attEDSEncQuery);
    
    pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString: ( !status ? @"true" : @"false" ) ];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
}




/**
 *
 * Get autoLoginInfo as cookie to use in the login request
 *
 */

- (void)autoLoginInfoAsCookie:(CDVInvokedUrlCommand *)command
{
    NSString *userName = [command argumentAtIndex:0];
    CDVPluginResult* pluginResult = nil;
    if (!userName) {
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"no parameter given"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:[command callbackId]];
        return;
    }
    
    
    NSDictionary* attEDSEncExistsQuery = @{
                                           (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
                                           (__bridge id)kSecAttrService : @"attEDSEnc",
                                           (__bridge id)kSecAttrAccount : userName,
                                           (__bridge id)kSecReturnData : @YES
                                           };
    
    CFDataRef attributes = nil;
    NSData* data = nil;
    OSStatus status = SecItemCopyMatching((__bridge CFDictionaryRef)attEDSEncExistsQuery, (CFTypeRef*)&attributes);
    
    NSString *secureCookie = nil;
    
    
    if (!status) {
        data = (__bridge_transfer NSData*)attributes;
        NSDictionary* _secData = [NSKeyedUnarchiver unarchiveObjectWithData:data];
        NSDictionary* result = [NSHTTPCookie requestHeaderFieldsWithCookies:@[
                                                                              [NSHTTPCookie cookieWithProperties:_secData]
                                                                              ]];
        secureCookie = [result valueForKey:@"Cookie"];
    }
    
    pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString: ( secureCookie ? secureCookie : @"" ) ];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
    
}


/**
 *
 * Check if auto login is supported, for iOS it is always supported
 *
 */

- (void)isAutoLoginSupported:(CDVInvokedUrlCommand *)command
{
        
    CDVPluginResult* pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString: @"true" ];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
}


/**
 *
 * Restore EDS secure info to the cookie store
 *
 */

- (void)addAutoLoginCookie:(CDVInvokedUrlCommand*)command
{
    
    NSString *userName = [command argumentAtIndex:0];
    // we don't realy need it in iOS since the domain is saved with the cookie, but just to keep it consistent with the java script interface
    NSString *cookieDomain = [command argumentAtIndex:1];
    
    CDVPluginResult* pluginResult = nil;
    if (!userName || !cookieDomain) {
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"no parameter given"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:[command callbackId]];
        return;
    }
    
    
    NSDictionary* attEDSEncExistsQuery = @{
                                           (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
                                           (__bridge id)kSecAttrService : @"attEDSEnc",
                                           (__bridge id)kSecAttrAccount : userName,
                                           (__bridge id)kSecReturnData : @YES
                                           };
    
    CFDataRef attributes = nil;
    NSData* data = nil;
    OSStatus status = SecItemCopyMatching((__bridge CFDictionaryRef)attEDSEncExistsQuery, (CFTypeRef*)&attributes);
    
    BOOL cookieRestored = NO;
    
    
    if (!status) {
        data = (__bridge_transfer NSData*)attributes;
        NSDictionary* _secData = [NSKeyedUnarchiver unarchiveObjectWithData:data];
        NSHTTPCookie *restoredCookie = [NSHTTPCookie cookieWithProperties:_secData];
        
        // just in case clear the cookie store from any existing attEDSEnc cookie
        
        NSHTTPCookie *storeCookie;
        NSHTTPCookieStorage *storage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
        for (storeCookie in [storage cookies]) {
            if ([[storeCookie name] isEqualToString:[restoredCookie name] ]  && [[storeCookie domain] isEqualToString:[restoredCookie domain]]) {
                [storage deleteCookie:storeCookie];
                [[NSUserDefaults standardUserDefaults] synchronize];
                break;
            }
        }
        
        
        // save the restored cookie back to store
        // store is set to NSHTTPCookieAcceptPolicyAlways on AppDelegate by cordova
        [storage setCookie:restoredCookie];
        cookieRestored = YES;
    }
    
    pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString: ( cookieRestored ? @"true" : @"false" ) ];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
}






@end
