package com.amdocs.keymanager;

import android.content.Context;
import android.content.SharedPreferences;
import android.security.KeyPairGeneratorSpec;
import android.util.Base64;

import java.math.BigInteger;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.util.Calendar;

import javax.crypto.Cipher;
import javax.security.auth.x500.X500Principal;

public class EncryptionUtils {

    public static final String ANDROID_KEYSTORE = "AndroidKeyStore";

    private static final String ALIAS = "imoc";

    private static KeyStore sStore;

    private static KeyStore getStore() {
        if (sStore == null) {
            try {
                sStore = KeyStore.getInstance(ANDROID_KEYSTORE);
                // Weird artifact of Java API.  If you don't have an InputStream to load, you still need
                // to call "load", or it'll crash.
                sStore.load(null);
            } catch (Exception e) {
                throw new RuntimeException("Unable to open key store", e);
            }
        }
        return sStore;
    }

    private static void generateNewKey(Context context) throws Exception {
        Calendar start = Calendar.getInstance();
        Calendar end = Calendar.getInstance();
        end.add(Calendar.YEAR, 10);

        KeyPairGeneratorSpec spec = new KeyPairGeneratorSpec.Builder(context)
            .setAlias(ALIAS)
            .setSubject(new X500Principal(String.format("CN=%s, OU=%s", ALIAS, context.getPackageName())))
            .setSerialNumber(BigInteger.TEN)
            .setStartDate(start.getTime())
            .setEndDate(end.getTime())
            .build();

        // use the Android key-store
        KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA", ANDROID_KEYSTORE);
        kpg.initialize(spec);

        // generates the key pair
        kpg.generateKeyPair();
    }

    public static String decode(String cipherText) throws Exception {
        final Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding", "AndroidOpenSSL");
        cipher.init(Cipher.DECRYPT_MODE, getPublicKey());
        return new String((cipher.doFinal(Base64.decode(cipherText, Base64.DEFAULT))));
    }

    public static String encode(String plainText) throws Exception {
        final Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding", "AndroidOpenSSL");
        cipher.init(Cipher.ENCRYPT_MODE, getPrivateKey());
        return Base64.encodeToString(cipher.doFinal(plainText.getBytes("UTF-8")), Base64.DEFAULT);
    }

    private static RSAPublicKey getPublicKey() throws Exception {
        KeyStore store = getStore();
        KeyStore.PrivateKeyEntry entry = (KeyStore.PrivateKeyEntry) store.getEntry(ALIAS, null);
        return (RSAPublicKey) entry.getCertificate().getPublicKey();
    }

    private static RSAPrivateKey getPrivateKey() throws Exception {
        KeyStore store = getStore();
        KeyStore.PrivateKeyEntry entry = (KeyStore.PrivateKeyEntry) store.getEntry(ALIAS, null);
        return (RSAPrivateKey) entry.getPrivateKey();
    }

     public static void init(Context ctx) {
        KeyStore store = getStore();
        try {
            KeyStore.Entry entry = store.getEntry(ALIAS, null);
            if (entry == null) {
                generateNewKey(ctx);
            } else if (!(entry instanceof KeyStore.PrivateKeyEntry)) {
                store.deleteEntry(ALIAS);
                generateNewKey(ctx);
            }
        } catch (Exception e) {
            throw new RuntimeException("Unable to initialize encryption", e);
        }
    }
}