
#import "CDVKeyManager.h"
#import "SFHFKeychainUtils.h"

@implementation CDVKeyManager

- (void) retrieve:(CDVInvokedUrlCommand*)command
{
    [self.commandDelegate runInBackground:^{
        NSArray* arguments = command.arguments;
        CDVPluginResult* pluginResult = nil;
        
        if ([arguments count] >= 2)
        {
            NSString* key = [arguments objectAtIndex:0];
            NSString* serviceName = [arguments objectAtIndex:1];
            NSError* error = nil;
            
            NSString* value = [SFHFKeychainUtils getPasswordForUsername:key andServiceName:serviceName error:&error];
            if (error == nil && value != nil) {
                pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:value];
            } else {
                pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                 messageAsString:[NSString stringWithFormat:@"error retrieving value for key '%@' : %@", key, [error localizedDescription]]];
            }
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                            messageAsString:@"incorrect number of arguments for retrieve Key"];
        }

        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void) store:(CDVInvokedUrlCommand*)command
{
    [self.commandDelegate runInBackground:^{
        NSArray* arguments = command.arguments;
        CDVPluginResult* pluginResult = nil;
        
        if ([arguments count] >= 3)
        {
            NSString* key = [arguments objectAtIndex:0];
            NSString* serviceName = [arguments objectAtIndex:1];
            NSString* value = [arguments objectAtIndex:2];
            NSError* error = nil;
            
            BOOL stored = [SFHFKeychainUtils storeUsername:key andPassword:value forServiceName:serviceName updateExisting:YES error:&error];
            if (stored && error == nil) {
                pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
            } else {
                pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:[error localizedDescription]];
            }
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                            messageAsString:@"incorrect number of arguments for storing Key"];
        }

        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void) remove:(CDVInvokedUrlCommand*)command
{
    [self.commandDelegate runInBackground:^{
        NSArray* arguments = command.arguments;
        CDVPluginResult* pluginResult = nil;
        
        if ([arguments count] >= 2)
        {
            NSString* key = [arguments objectAtIndex:0];
            NSString* serviceName = [arguments objectAtIndex:1];
            NSError* error = nil;
            
            BOOL deleted = [SFHFKeychainUtils deleteItemForUsername:key andServiceName:serviceName error:&error];
            if (deleted && error == nil) {
                pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
            } else {
                pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:[error localizedDescription]];
            }
        }
        else
        {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                            messageAsString:@"incorrect number of arguments for removing Key"];
        }

        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

@end
